precision mediump float;


varying vec4 v_fragmentColor;
varying vec2 v_texCoord;


uniform vec3 blendColor;
uniform float factor;


vec3 RGBToHSL(vec3 color)
{
	vec3 hsl;

	float fmin = min(min(color.x, color.y), color.z);
	float fmax = max(max(color.x, color.y), color.z);
	float delta = fmax - fmin;

	hsl.z = (fmax + fmin) / 2.0; // Luminance

	if (delta == 0.0)		//This is a gray, no chroma...
	{
		hsl.x = 0.0;	// Hue
		hsl.y = 0.0;	// Saturation
	}
	else                                    //Chromatic data...
	{
		if (hsl.z < 0.5)
			hsl.y = delta / (fmax + fmin); // Saturation
		else
			hsl.y = delta / (2.0 - fmax - fmin); // Saturation

		float deltaR = (((fmax - color.x) / 6.0) + (delta / 2.0)) / delta;
		float deltaG = (((fmax - color.y) / 6.0) + (delta / 2.0)) / delta;
		float deltaB = (((fmax - color.z) / 6.0) + (delta / 2.0)) / delta;

		if (color.x == fmax )
			hsl.x = deltaB - deltaG; // Hue
		else if (color.y == fmax)
			hsl.x = (1.0 / 3.0) + deltaR - deltaB; // Hue
		else if (color.z == fmax)
			hsl.x = (2.0 / 3.0) + deltaG - deltaR; // Hue

		if (hsl.x < 0.0)
			hsl.x += 1.0; // Hue
		else if (hsl.x > 1.0)
			hsl.x -= 1.0; // Hue
	}

	return hsl;
}

float HueToRGB(float f1, float f2, float hue)
{
	if (hue < 0.0)
		hue += 1.0;
	else if (hue > 1.0)
		hue -= 1.0;
	float res;
	if ((6.0 * hue) < 1.0)
		res = f1 + (f2 - f1) * 6.0 * hue;
	else if ((2.0 * hue) < 1.0)
		res = f2;
	else if ((3.0 * hue) < 2.0)
		res = f1 + (f2 - f1) * ((2.0 / 3.0) - hue) * 6.0;
	else
		res = f1;
	return res;
}

vec3 HSLToRGB(vec3 hsl)
{
	vec3 rgb;

	if (hsl.y == 0.0)
		rgb = vec3(hsl.z, hsl.z, hsl.z); // Luminance
	else
	{
		float f2;

		if (hsl.z < 0.5)
			f2 = hsl.z * (1.0 + hsl.y);
		else
			f2 = (hsl.z + hsl.y) - (hsl.y * hsl.z);

		float f1 = 2.0 * hsl.z - f2;

		rgb.r = HueToRGB(f1, f2, hsl.x + (1.0/3.0));
		rgb.g = HueToRGB(f1, f2, hsl.x);
		rgb.b= HueToRGB(f1, f2, hsl.x - (1.0/3.0));
	}

	return rgb;
}


void main()
{
    vec4 texColor;
    if(CC_TextureAlpha == 0.0) {
        texColor = texture2D(CC_Texture0, v_texCoord);
    } else {
        texColor = vec4(texture2D(CC_Texture0, v_texCoord).rgb, texture2D(CC_Texture1, v_texCoord).r);
        texColor.rgb *= texColor.a; // premultiply alpha channel
    }
	vec4 textureColor = texColor;
	vec3 base = vec3(textureColor.r,textureColor.g,textureColor.b);
	vec3 blendHSL = RGBToHSL(blendColor);
    vec3 baseColor =  HSLToRGB(vec3(blendHSL.x, blendHSL.y, RGBToHSL(base).z));

    baseColor.r = (baseColor.r - textureColor.r)*factor + textureColor.r;
    baseColor.g = (baseColor.g - textureColor.g)*factor + textureColor.g;
    baseColor.b = (baseColor.b - textureColor.b)*factor + textureColor.b;

 	vec4 target_color = v_fragmentColor * vec4(baseColor.r , baseColor.g , baseColor.b , textureColor.a);
	gl_FragColor = target_color;
}