precision mediump float;
varying vec4 v_fragmentColor;
varying vec2 v_texCoord;

uniform vec4 u_GradColorUp;
uniform vec4 u_GradColorDown;
uniform vec4 u_outlineColor;
uniform vec2 u_textureSize;

const float outlineSize = 1.0;
float saturate(float x)
{
  return max(0.0, min(1.0, x));
}

vec3 lerp_rgb(vec3 from, vec3 to, float t){
	return from + (to - from)* t;
}

//const float cosArray[12]={1.0, 0.866, 0.5, 0.0, -0.5, -0.866, -0.1, -0.866, -0.5, 0.0, 0.5, 0.866};
//const float sinArray[12]={0.0, 0.5, 0.866, 1.0, 0.866, 0.5, 0.0, -0.5, -0.866, -1.0, -0.866, -0.5};

int getIsStrokeWithAngel(float angel)
{
	int stroke = 0;
	float rad = angel * 0.01745329252;
	vec2 uv_dst = vec2(v_texCoord.x + outlineSize * cos(rad) / u_textureSize.x, v_texCoord.y + outlineSize * sin(rad) / u_textureSize.y);
	float a = texture2D(CC_Texture0, uv_dst).a;
	if (a >= 0.01)
	{
		stroke = 1;
	}
	return stroke;
}

void main()
{
	//gradient it
	vec4 src_color = texture2D(CC_Texture0, v_texCoord).rgba;
	vec4 font_color = src_color;
	vec4 font_outline = src_color;
	float uv_offset = -0.22; //Range(-10 , 10)
	float soften = 0.25;  	 //Range(0.1 , 10)
	float brightness = 0.0;	 //Range(0 , 1)
	if (src_color.a > 0.01)
	{
		float clampResult55 = clamp(saturate(((v_texCoord.y + uv_offset) / soften)) , brightness , 1.0); 
		vec3 lerpResult61 = lerp_rgb(u_GradColorDown.rgb, u_GradColorUp.rgb, clampResult55);		
		font_color = vec4(lerpResult61, src_color.a);
	}
	
	int strokeCount = 0;
	strokeCount += getIsStrokeWithAngel(0.0);
	strokeCount += getIsStrokeWithAngel(30.0);
	strokeCount += getIsStrokeWithAngel(60.0);
	strokeCount += getIsStrokeWithAngel(90.0);
	strokeCount += getIsStrokeWithAngel(120.0);
	strokeCount += getIsStrokeWithAngel(150.0);
	strokeCount += getIsStrokeWithAngel(180.0);
	strokeCount += getIsStrokeWithAngel(210.0);
	strokeCount += getIsStrokeWithAngel(240.0);
	strokeCount += getIsStrokeWithAngel(270.0);
	strokeCount += getIsStrokeWithAngel(300.0);
	strokeCount += getIsStrokeWithAngel(330.0);
	if (strokeCount > 0)
	{
		font_outline.rgb = u_outlineColor.rgb;				
		font_outline.a = 1.0;
	}
	
	gl_FragColor = vec4(font_color.rgb * font_color.a + font_outline.rgb * (1.0-font_color.a), font_outline.a);
}
