//
// Created by 王银报 on 2018/5/29.
//

#include  "jin_assist.h"
#include "jsonxx.h"
#include "jni_utils.h"
#include "llh_uni_sdk_export.h"

namespace  LLHUni
{

    JniAssist::JniAssist() {

    }

    JniAssist::~JniAssist() {

    }

    JniAssist& JniAssist::GetInstance()
    {

        static JniAssist instance = JniAssist();
        return instance;
    }



    void JniAssist::setJVM(JavaVM *jvm)
    {
        _jvm = jvm;
    }

    void JniAssist::ExecuteJavaStaticVoidMethod(const char *class_name, const char *method_name,
                                                const char *sig,
                                                std::function<void(JNIEnv *, jclass,
                                                                   jmethodID)> executor) {
        if (!method_name || !sig)
        {
            return;
        }

        JNIEnv *env = GetJniEnv();
        jclass clazz = GetClassByName(class_name);

        if (clazz)
        {
            jmethodID method_id = env->GetStaticMethodID(clazz, method_name, sig);
            if (method_id)
            {
                if (executor)
                {
                    executor(env, clazz, method_id);
                }
            }
        }
    }

    JNIEnv *JniAssist::GetJniEnv() const {
        if (_jvm)
        {
            JNIEnv *env;
            _jvm ->GetEnv((void **)(&env), JniVerison);
            return env;
        }
        return nullptr;
    }

    jclass JniAssist::GetClassByName(const char *class_name) {
        if (!class_name)
        {
            return  NULL;
        }
        JNIEnv *env = GetJniEnv();
        jclass clasz = m_jni_class_map_ [class_name];
        if (!clasz)
        {
            clasz = env->FindClass(class_name);
            if (clasz)
            {
                m_jni_class_map_[class_name] = static_cast<jclass >(env->NewGlobalRef(clasz));

            }
        }
        return clasz;
    }



    void JniAssist::RegisterLLHUniInitCallback(LLHUniInitCallback callback) {
        if (callback)
        {
            m_init_callback_ptr_ = std::make_shared<LLHUniInitCallback >(callback);
        }
    }


    void JniAssist::RegisterLLHUniLoginCallback(LLHUniLoginCallback callback) {
        if (callback)
        {
            m_login_callback_ptr_ = std::make_shared<LLHUniLoginCallback >(callback);
        }
    }

    void JniAssist::RegisterLLHUniPayCallback(LLHUniPayCallback callback) {
        if (callback)
        {
            m_pay_callback_ptr_ = std::make_shared<LLHUniPayCallback >(callback);
        }
    }

    void JniAssist::RegisterLLHUniExitCallback(LLHUniExitCallback callback) {
        if (callback)
        {
            m_exit_callback_ptr_ = std::make_shared<LLHUniExitCallback >(callback);
        }
    }

    void JniAssist::RegisterLLHUniPermissionRequestCallback(LLHUniPermissionRequestCallback callback) {
        if (callback)
        {
            m_permission_callback_ptr_ = std::make_shared<LLHUniPermissionRequestCallback>(callback);
        }
    }



    LLHUniInitCallback JniAssist::GetLLHUniInitCallback() {
        return *m_init_callback_ptr_;
    }

    LLHUniLoginCallback JniAssist::GetLLHUniLoginCallback() {
        return *m_login_callback_ptr_;
    }

    LLHUniPayCallback JniAssist::GetLLHUniPayCallback() {
        return *m_pay_callback_ptr_;
    }

    LLHUniExitCallback JniAssist::GetLLHUniExitCallback() {
        return *m_exit_callback_ptr_;
    }

    LLHUniPermissionRequestCallback JniAssist::GetLLHUniPermissionRequestCallback() {
        return *m_permission_callback_ptr_;
    }






    LLHUniPayRequest *JniAssist::JsonToLLHUniPayRequest(const char *c_str) {
        return nullptr;
    }

    LLHUniUser *JniAssist::JsonToLLHUniUser(const char *c_str) {
        if(c_str)
        {
            std::string str(c_str);
            jsonxx::Object obj;
            obj.parse(str);

            LLHUniUser *user = new LLHUniUser();
            if (obj.has<jsonxx::String>("id"))
            {
                user->id = JniUtils::RetainCStr(obj.get<jsonxx::String>("id").c_str());
            }

            if (obj.has<jsonxx::String>("token"))
            {
                user->token = JniUtils::RetainCStr(obj.get<jsonxx::String>("token").c_str());
            }

            if (obj.has<jsonxx::String>("user_name"))
            {
                user->user_name = JniUtils::RetainCStr(obj.get<jsonxx::String>("user_name").c_str());
            }

            if (obj.has<jsonxx::String>("avatar_url"))
            {
                user->avatar_url = JniUtils::RetainCStr(obj.get<jsonxx::String>("avatar_url").c_str());
            }
            return user;
        }

        return nullptr;
    }

    const char **JniAssist::JSONToCStrArray(const char *c_str, size_t &size) {
        if (c_str)
        {
            std::string str(c_str);
            jsonxx::Array array;
            array.parse(str);

            size = array.size();

            if (size > 0)
            {
                const char *str_array[size];
                for (size_t i = 0; i <size; ++i)
                {
                    str_array[i] = JniUtils::RetainCStr(array.get<jsonxx::String>(i).c_str());
                }
                return str_array;
            }


        }
        return nullptr;
    }

    const char *JniAssist::LLHUniPayRequestToCStr(LLHUniPayRequest *request) {

        if(request)
        {
            jsonxx::Object obj = jsonxx::Object();
            if (request->order_id)
            {
                obj << "order_id" << std::string(request->order_id);
            }

            if (request->price)
            {
                obj <<"price" << request->price;
            }
            if (request->exchange_rate)
            {
                obj <<"exchange_rate" <<request->exchange_rate;
            }

            if (request->item_id)
            {
                obj <<"item_id" <<request->item_id;
            }

            if (request->item_id_serial)
            {
                obj <<"item_id_serial" <<request->item_id_serial;
            }
            if (request->item_name)
            {
                obj<<"item_name" <<request->item_name;
            }

            if (request->item_desc)
            {
                obj << "item_desc" << std::string(request->item_desc);
            }
            if (request->coin_icon_path)
            {
                obj << "coin_icon_path" << std::string(request->coin_icon_path);
            }
            if (request->extension)
            {
                obj << "extension" << std::string(request->extension);
            }
            if (request->internal_item_id)
            {
                obj << "internal_item_id" << std::string(request->internal_item_id);
            }
            return JniUtils::RetainCStr(obj.json().c_str());
        }

        return nullptr;
    }

     const char* JniAssist::LLHUniGameInfoToCStr(LLHUniGameInfo *gameInfo) {

         if (gameInfo)
         {
             jsonxx::Object obj = jsonxx::Object();
             if (gameInfo->role_id)
             {
                 obj<< "role_id" << std::string(gameInfo->role_id);
             }
             if (gameInfo->role_name)
             {
                 obj<< "role_name" <<std::string(gameInfo->role_name);
             }
             if (gameInfo->level)
             {
                 obj<< "level" <<gameInfo->level;
             }

             if (gameInfo->vip_level)
             {
                 obj<<"vip_level"<< gameInfo->vip_level;
             }
             if (gameInfo->server_id)
             {
                 obj<<"server_id"<<gameInfo->server_id;
             }
             if (gameInfo->server_name)
             {
                 obj<<"server_name"<<std::string(gameInfo->server_name);
             }

             if (gameInfo->guild_id)
             {
                 obj<<"guild_id"<<gameInfo->guild_id;
             }

             if (gameInfo->guild)
             {
                 obj<<"guild" <<std::string(gameInfo->guild);
             }
             if (gameInfo->balance)
             {
                 obj<<"balance"<<gameInfo->balance;
             }
             if (gameInfo->balance_unit)
             {
                 obj<<"balance_unit"<<std::string(gameInfo->balance_unit);
             }
             if (gameInfo->role_create_time)
             {
                 obj<<"role_create_time"<<gameInfo->role_create_time;
             }
             if (gameInfo->role_level_change_time)
             {
                 obj<<"role_level_change_time"<<gameInfo->role_level_change_time;
             }
         }

         return NULL;
     }

}