//
// Created by 王银报 on 2018/5/29.
//

#ifndef LILITHSDKPROJ_JIN_ASSIST_H
#define LILITHSDKPROJ_JIN_ASSIST_H
#include "jni.h"
#include "llh_uni_sdk_export.h"
#include <functional>
#include <map>
namespace LLHUni
{
    class JniAssist
    {
    public:
        JniAssist();
        virtual  ~JniAssist();

    public:
        static JniAssist &GetInstance();
        const  jint JniVerison = JNI_VERSION_1_6;
        void setJVM(JavaVM *jvm);





    public:

        void ExecuteJavaStaticVoidMethod(const char *class_name, const char *method_name
                , const char *sig, std::function<void (JNIEnv *, jclass , jmethodID)> executor);

        template  <typename T>
        T ExecuteJavaStaticMethod(const char *class_name,
                                                             const char *method_name,
                                                             const char* sig, T default_value,
                                                             std::function<T(JNIEnv* , jclass, jmethodID)> executor)
        {
            if (!method_name || sig)
            {
                return  default_value;
            }

            JNIEnv *env = GetJniEnv();
            jclass clasz = GetClassByName(class_name);
            if (clasz)
            {
                jmethodID method_id = env->GetStaticMethodID(clasz, method_name, sig);
                if (method_id )
                {
                    if (executor)
                    {
                        return  executor(env, clasz, method_id);
                    }
                }
            }

            return default_value;

        }


    private:
        JavaVM *_jvm;
        std::map<const char *, jclass > m_jni_class_map_ {std::map<const char*, jclass>()};
        JNIEnv *GetJniEnv() const;
        jclass  GetClassByName(const char *class_name);

    public:

        void RegisterLLHUniInitCallback(LLHUniInitCallback callback);
        void RegisterLLHUniLoginCallback(LLHUniLoginCallback  callback);
        void RegisterLLHUniPayCallback(LLHUniPayCallback callback);
        void RegisterLLHUniExitCallback(LLHUniExitCallback callback);
        void RegisterLLHUniPermissionRequestCallback(LLHUniPermissionRequestCallback callback);



        LLHUniInitCallback  GetLLHUniInitCallback();
        LLHUniLoginCallback  GetLLHUniLoginCallback();
        LLHUniPayCallback  GetLLHUniPayCallback();
        LLHUniExitCallback  GetLLHUniExitCallback();
        LLHUniPermissionRequestCallback  GetLLHUniPermissionRequestCallback();


    private:

        std::shared_ptr<LLHUniInitCallback> m_init_callback_ptr_ {std::make_shared<LLHUniInitCallback >()};
        std::shared_ptr<LLHUniLoginCallback> m_login_callback_ptr_ {std::make_shared<LLHUniLoginCallback >()};
        std::shared_ptr<LLHUniPayCallback> m_pay_callback_ptr_ {std::make_shared<LLHUniPayCallback>()};
        std::shared_ptr<LLHUniExitCallback> m_exit_callback_ptr_ {std::make_shared<LLHUniExitCallback>()};
        std::shared_ptr<LLHUniPermissionRequestCallback > m_permission_callback_ptr_ {std::make_shared<LLHUniPermissionRequestCallback >()};

    public:
        LLHUniUser *JsonToLLHUniUser(const char *c_str);
        LLHUniPayRequest *JsonToLLHUniPayRequest(const char *c_str);
        const char **JSONToCStrArray(const char *c_str, size_t &size);
        const char *LLHUniPayRequestToCStr(LLHUniPayRequest *request);
        const char *LLHUniGameInfoToCStr(LLHUniGameInfo *gameInfo);
    };



}

#endif //LILITHSDKPROJ_JIN_ASSIST_H
