//
// Created by 王银报 on 2018/6/4.
//

#include "jni_utils.h"

namespace  LLHUni
{



    const char *LLHUni::JniUtils::RetainCStr(const char *str) {
        if (!str)
        {
            return  NULL;
        }

        char *c_str = (char *) malloc(strlen(str)+1);
        strcpy(c_str, str);
        return (const char *) c_str;
    }

    std::string LLHUni::JniUtils::JStrToStr(JNIEnv *env, jstring jstr) {
        const char *cstr = JniUtils::JStrToCStr(env, jstr);
        if (! jstr)
        {
            return std::string();
        }

        std::string str(cstr);
        env->ReleaseStringUTFChars(jstr, cstr);
        return str;

    }

    const char *LLHUni::JniUtils::JStrToCStr(JNIEnv *env, jstring jstr) {

        if (!jstr)
        {
            return NULL;
        }
        const char *cstr = env->GetStringUTFChars(jstr, JNI_FALSE);
        return cstr;
    }

    jstring LLHUni::JniUtils::StrToJStr(JNIEnv *env, const std::string &str) {
        return  env->NewStringUTF(str.c_str());
    }

    jstring LLHUni::JniUtils::CStrToJStr(JNIEnv *env, const char *str) {
        if (str)
        {
            return env->NewStringUTF(str);
        }
        return NULL;
    }

    jbyteArray LLHUni::JniUtils::StrToJByteArray(JNIEnv *env, const std::string &str) {
        jbyteArray buffer = env->NewByteArray(static_cast<jsize >(str.size()));
        env->SetByteArrayRegion(buffer, 0, static_cast<jsize >(str.size()), (jbyte *)str.c_str());
        return buffer;
    }

    jbyteArray LLHUni::JniUtils::CharArrryToJByteArray(JNIEnv *env, const char *charArray, int len) {
        if (charArray)
        {
            jbyteArray array = env->NewByteArray(len);
            env->SetByteArrayRegion(array, 0, len, (jbyte *) charArray);
            return array;
        }
        return nullptr;
    }

    int LLHUni::JniUtils::GetByteArrayLen(JNIEnv *env, jarray jArray) {

        if (jArray)
        {
            return  env->GetArrayLength(jArray);
        }
        return 0;
    }

    jobjectArray LLHUni::JniUtils::CStrArrayToJobjectArray(JNIEnv *env, const char **strs, size_t length) {

        static jclass s_str_class = static_cast<jclass> (env->NewGlobalRef(env ->FindClass("java/lang/String")));
        if (length ==0  || !strs)
        {
            return  NULL;
        }
        jobjectArray  array = env ->NewObjectArray(static_cast<jsize >(length), s_str_class, 0);
        for (size_t i =0; i <length; ++i)
        {
            env->SetObjectArrayElement(array, static_cast<jsize >(i), CStrToJStr(env, strs[i]));
        }
        return array;
    }

    const char *LLHUni::JniUtils::JByteArrayToCharArray(JNIEnv *env, jbyteArray byteArray) {

        if (byteArray)
        {
            return  (char*)env->GetByteArrayElements(byteArray, JNI_FALSE);
        }
        return nullptr;
    }
}