//
// Created by 王银报 on 2018/6/1.
//


#include "llh_uni_sdk_export.h"
#include "jin_assist.h"
#include "jni_utils.h"


using namespace LLHUni;
const char * const LLH_UNI_JNI_CLASS = "com/lilith/sdk/uni/inde/UniIndeJni";


#ifndef LLH_UNI_V_RV_CALL
#define LLH_UNI_V_RV_CALL(METHOD_NAME) \
JniAssist::GetInstance().ExecuteJavaStaticVoidMethod( \
LLH_UNI_JNI_CLASS, \
(METHOD_NAME), \
"()V", \
[](JNIEnv *env, jclass clazz, jmethodID method) \
{ \
env->CallStaticVoidMethod(clazz, method); \
})
#endif

#ifndef LLH_UNI_V_RI32_CALL
#define LLH_UNI_V_RI32_CALL(METHOD_NAME,DEFAULT_RET) \
JniAssist::GetInstance().ExecuteJavaStaticMethod<int32_t>( \
LLH_UNI_JNI_CLASS, \
(METHOD_NAME), \
"()I", \
(DEFAULT_RET), \
[](JNIEnv *env, jclass clazz, jmethodID method) \
{ \
return static_cast<int32_t>(env->CallStaticIntMethod(clazz, method)); \
})
#endif

#ifndef LLH_UNI_V_RZ_CALL
#define LLH_UNI_V_RZ_CALL(METHOD_NAME,DEFAULT_RET) \
JniAssist::GetInstance().ExecuteJavaStaticMethod<bool>( \
LLH_UNI_JNI_CLASS, \
(METHOD_NAME), \
"()Z", \
(DEFAULT_RET), \
[](JNIEnv *env, jclass clazz, jmethodID method) \
{ \
return static_cast<bool>(env->CallStaticBooleanMethod(clazz, method)); \
})
#endif

#ifndef LLH_UNI_V_RSTRING_CALL
#define LLH_UNI_V_RSTRING_CALL(METHOD_NAME,DEFAULT_RET) \
JniAssist::GetInstance().ExecuteJavaStaticMethod<const char *>( \
LLH_UNI_JNI_CLASS, \
(METHOD_NAME), \
"()Ljava/lang/String;", \
(DEFAULT_RET), \
[](JNIEnv *env, jclass clazz, jmethodID method) \
{ \
jstring jstr = static_cast<jstring>(env->CallStaticObjectMethod(clazz, method)); \
const char *c_str = JniUtils::JStrToCStr(env, jstr); \
const char *retain_c_str = JniUtils::RetainCStr(c_str); \
env->ReleaseStringUTFChars(jstr, c_str); \
env->DeleteLocalRef(jstr); \
return retain_c_str; \
})
#endif



void LLHUniRegisterInitCallback(LLHUniInitCallback callback)
{
    if (callback)
    {
        JniAssist::GetInstance().RegisterLLHUniInitCallback(std::move(callback));
    }
}


void LLHUniRegisterLoginCallbck(LLHUniLoginCallback callback)
{
    if (callback)
    {
        JniAssist::GetInstance().RegisterLLHUniLoginCallback(std::move(callback));
    }

}

void LLHUniRegisterPayCallback(LLHUniPayCallback callback)
{
    if (callback)
    {
        JniAssist::GetInstance().RegisterLLHUniPayCallback(std::move(callback));
    }

}

void LLHUniRegisterExitCallback(LLHUniExitCallback callback)
{
    if (callback)
    {
        JniAssist::GetInstance().RegisterLLHUniExitCallback(std::move(callback));
    }
}

void LLHUniRegisterPermissionRequestCallback(LLHUniPermissionRequestCallback callback) {
    if (callback)
    {
        JniAssist::GetInstance().RegisterLLHUniPermissionRequestCallback(std::move(callback));
    }
}

const char *LLHUniGetType() {
    return LLH_UNI_V_RSTRING_CALL("getType", NULL);
}

const char *LLHUniGetThirdPartyVerison() {
    return LLH_UNI_V_RSTRING_CALL("getThirdPartyVersion", NULL);
}


void LLHUniPay(LLHUniPayRequest *pay_request) {

    JniAssist::GetInstance().ExecuteJavaStaticVoidMethod(
            LLH_UNI_JNI_CLASS,
            "pay",
            "(Ljava/lang/String;)V",
            [pay_request](JNIEnv *env, jclass clazz, jmethodID method)
            {
                const  char *str = JniAssist::GetInstance().LLHUniPayRequestToCStr(pay_request);
                env->CallStaticVoidMethod(clazz, method, JniUtils::CStrToJStr(env, str));
            }
    );
}

bool LLHUniIsPayValid() {
    return LLH_UNI_V_RZ_CALL("isPayValid", false);
}

void LLHUniPayWithApplyOrderId(LLHUniPayRequest *pay_request) {
    JniAssist::GetInstance().ExecuteJavaStaticVoidMethod(
            LLH_UNI_JNI_CLASS,
            "applyOrderPay",
            "(Ljava/lang/String;)V",
            [pay_request](JNIEnv *env, jclass clazz, jmethodID method)
            {
                const  char *str = JniAssist::GetInstance().LLHUniPayRequestToCStr(pay_request);
                env->CallStaticVoidMethod(clazz, method, JniUtils::CStrToJStr(env, str));
            }
    );

}

void LLHUniShareImageToThird(const char *path) {
    JniAssist::GetInstance().ExecuteJavaStaticVoidMethod(
            LLH_UNI_JNI_CLASS,
            "shareImage",
            "(Ljava/lang/String;)V",
            [path](JNIEnv *env, jclass clazz, jmethodID method)
            {
                env->CallStaticVoidMethod(clazz, method, JniUtils::CStrToJStr(env, path));
            }
    );

}

void LLHUniReportGameInfo(uint32_t eventType, LLHUniGameInfo *gameInfo) {
    const char* str_game = JniAssist::GetInstance().LLHUniGameInfoToCStr(gameInfo);

    jint event = static_cast<jint >(eventType);
    JniAssist::GetInstance().ExecuteJavaStaticVoidMethod(
            LLH_UNI_JNI_CLASS,
            "reportGameInfo",
            "(Ljava/lang/String;)V",
            [event, str_game](JNIEnv *env, jclass clazz, jmethodID method)
            {
                env->CallStaticVoidMethod(clazz,
                                          method,
                                          event,
                                          JniUtils::CStrToJStr(env, str_game));

            }
    );
}

bool LLHUniIsSwichAccountValid() {
    return LLH_UNI_V_RZ_CALL("isSwitchAccountValid", false);
}

void LLHUniSwichAccount() {
    LLH_UNI_V_RV_CALL("switchAccount");
}


bool LLHUniIsExitValid() {
    return LLH_UNI_V_RZ_CALL("isExitValid", false);
}

void LLHUniExit() {
    LLH_UNI_V_RV_CALL("exit");
}

bool LLHUniIsQuitLoginValid() {
    return LLH_UNI_V_RZ_CALL("isQuitLoginValid", false);
}

void LLHUniQuitLogin() {
    LLH_UNI_V_RV_CALL("quitLogin");
}

const char *LLHUniGetChannelID() {
    return LLH_UNI_V_RSTRING_CALL("getChannelId", NULL);
}

bool LLHUniIsLoginValid() {
    return LLH_UNI_V_RZ_CALL("isLoginValid", false);
}

void LLHIndeCheckPermissions() {
    LLH_UNI_V_RV_CALL("checkPremission");
}

bool LLHUniLogin() {
    LLH_UNI_V_RV_CALL("login");
}


LLHUniUser *LLHUniGetCurrentUser() {
   const char * str = LLH_UNI_V_RSTRING_CALL("getCurrentUser", NULL);
    LLHUniUser *user = JniAssist::GetInstance().JsonToLLHUniUser(str);
    free(const_cast<char *> (str));
    return user;
}

LLHUniUser *LLHUniGetLastUser() {
    const char * str = LLH_UNI_V_RSTRING_CALL("getLastUser", NULL);
    LLHUniUser *user = JniAssist::GetInstance().JsonToLLHUniUser(str);
    free(const_cast<char *> (str));
    return user;
}

void LLHUniStartAuthenticationInfo() {
    LLH_UNI_V_RV_CALL("startAuthenticationInfo");
}















